package fwpUtil;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Title: Color chooser dialog.</p>
 * <p>
 * Description: It integrates the fairly standard color chooser
 * which allows specifying the color by using swatches, HSB or RGB tabs with
 * an added alpha/transparency input.
 * <p>
 * Copyright: Copyright (c) 2011
 * @author Fred Pospeschil - This file is placed in the public domain
 * and can be used and adapted without restrictions.  The user of the code is
 * totally responsible for insuring its suitability in the intended application.
 * @version 1.0 - 2011
  * <p>
 */


public class ColorChooser extends JDialog
  {

  private JColorChooser  chooser;
  private Color          newColor      = new Color(0,0,0,155); // Black
  private Color          newAlphaColor = new Color(0,0,0,255);
  private JTextField     alphaTextField = new JTextField();
  private Frame          parent;

  /**
   * Displays a dialog in which the user can select a color and an alpha
   * (transparency) value.
   * <p>
   * A color chooser that allows users to pick a color in three different
   * manners.  The color selection is provided by a JColorChooser pane.
   * <p>
   * The dialog Can be called with
   * <code>ColorChooser fc = new ColorChooser(this);</code>
   * if a frame is avialable or it can be called with:
   * <code>ColorChooser fc = new ColorChooser(null);</code> if a frame is not
   * available.
   * <p>
   * If a frame is provided, the dialog is centered over that frame.  Otherwise,
   * if a <code>null</code> is passed, the dialog is positioned 20 pixels down
   * and to the right of the upper left corner of the display screen.
   * <p>
   * This dialog is an adaptation of the font chooser presented in Java Swing by
   * Robert Eckstein, Marc Loy & Dave Wood, O'Reilly, Sebastopol, CA 1998,
   * ISBN 1-56592-455-X
   * @param parent The Frame from which the dialog was called.
   */
  public ColorChooser(Frame parent)
    {
    super(parent, "Color Chooser", true);
    this.parent = parent;

    // make sure that any way they cancel the window does the right thing
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        closeAndCancel(); }});

    // Start the long process of setting up our interface
    Container c = getContentPane();

    // Set up the color chooser panel and attach a change listener so that color
    // updates get reflected in our preview label.
    chooser = new JColorChooser(Color.black);
    c.add(chooser, BorderLayout.CENTER);

    JPanel previewPanel = new JPanel(new BorderLayout());

    // Add in the Ok and Cancel buttons for our dialog box
    JButton okButton = new JButton("Ok");
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        closeAndSave(); } });

    JButton cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        closeAndCancel(); } });

    JLabel        alphaLabel     = new JLabel("Alpha (1-255)");
    alphaTextField.setText("255");
    alphaTextField.setColumns(3);
    alphaTextField.setActionCommand("alphaTextField");

    JPanel controlPanel = new JPanel();
    controlPanel.add(okButton);
    controlPanel.add(cancelButton);
    controlPanel.add(alphaLabel);
    controlPanel.add(alphaTextField);
    previewPanel.add(controlPanel, BorderLayout.SOUTH);

    // Give the preview label room to grow.
    previewPanel.setMinimumSize(new Dimension(100, 35));
    previewPanel.setPreferredSize(new Dimension(100, 35));

    c.add(previewPanel, BorderLayout.SOUTH);

   // Position the dialog
   if (parent == null)
     { // If there is no parent then position the dialog at the upper left
     this.setLocation(20, 20);
     }
   else
     { // Else position the dialog over the center of the parent
     Dimension dlgSize = this.getPreferredSize();
     Dimension frmSize = parent.getSize();
     Point loc = parent.getLocation();
     this.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                      (frmSize.height - dlgSize.height) / 2 + loc.y);
     }
  pack();
  this.setVisible(true);
  } // End ColorChooser constructor


  private void closeAndSave()
    {
    // Get the color the user slelected
    newColor  = chooser.getColor();
    // Extract the basic colors for use in building the color with alpha.
    int red   = newColor.getRed();
    int green = newColor.getGreen();
    int blue  = newColor.getBlue();
    // Now build new color using the rgb and alpha values
    int newAlpha = 255;
    String alpha = alphaTextField.getText();
    newAlpha = Integer.valueOf(alpha);
    // The alpha value must be between 1 and 255
    if (newAlpha < 1 || newAlpha > 255)
      {
      newAlpha = 255;
      alphaTextField.setText("255");
      JOptionPane.showMessageDialog(parent,
             "The alpha transparency value must be between 1 and 255." +
           "\n        The program has reset the alpha to 255.",
           "The alpha value was in error.",
           JOptionPane.WARNING_MESSAGE);
      }

    newAlphaColor = new Color(red,green,blue,newAlpha);
    setVisible(false);
    }

  private void closeAndCancel()
    { setVisible(false); }

  /**
   * Returns the color that was selected by the user.
   *
   * @return A Color object with red, green, blue and alpha set.  The
   * alpha/transparency value is constrained to fall beween (inclusive)
   * 1 and 255.  If an invalid value is entered the program advises the
   * user and sets the value to 255.  The user then has the opportunity to
   * enter another legal value.
   * <p>
   * Note: The alpha values set by the user do not change the transparency of
   * the text shown by the color chooser but are returned in the color object.
   */
  public Color getNewColor()
    { return newAlphaColor; }

  /**
   * @return Provides a string representation of the selected color and color-alpha.
   * <p>
   * Example: java.awt.Color[r=255,g=51,b=51]Alpha=179
   */
  public String toString()
    { return newAlphaColor.toString() + "Alpha="+newAlphaColor.getAlpha(); }

  } // End ColorChooser
