package fwpUtil;


// FontAndColorChooser.java
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.colorchooser.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import fwpUtil.*;

/**
 * Title: Font and color chooser dialog.</p>
 * <p>
 * Description: Displays a dialog in which the user can select a font and its
 * color, bold, italic and transparency attributes.
 * <p>
 * Copyright: Copyright (c) 2011
 *
 * @author Fred Pospeschil - This file is placed in the public domain
 * and can be used and adapted without restrictions.  The user of the code is
 * totally responsible for insuring its suitability in the intended application.
 * <p>
 * This dialog is an adaptation of the font chooser presented in Java Swing by
 * Robert Eckstein, Marc Loy & Dave Wood, O'Reilly, Sebastopol, CA 1998,
 * ISBN 1-56592-455-X. See pages 381-387.  The fixed list of fonts approach
 * used by the O'Reilly example was replaced by the more general getting a
 * list of all available fonts as described on pages 788-791 of Java for
 * Dummies by doug Lowe and Barry Burd, Wiley Publishing, Inc. 2007.
 * @version 1.0 - 2011
 */

public class FontAndColorChooser extends JDialog implements ActionListener
  {

  private JColorChooser      colorChooser;
  private JComboBox          fontName;
  private JCheckBox          fontBold, fontItalic;
  private JTextField         fontSize;
  private JLabel             previewLabel;
  private SimpleAttributeSet attributes;
  private Font               newFont;
  private Color              newColor;
  private Color              newAlphaColor = new Color(0,0,0,255);
  private JTextField         alphaTextField = new JTextField();
  private Frame              parent;


  /**
 * Displays a dialog in which the user can select a font and its color,
 * bold, italic and transparendcy attributes.
 * <p>
 * If a Frame, or Frame subclass, is avialable call with:
 * <p>
 * <code>FontAndColorChooser fc = new FontAndColorChooser(this, true/false);</code>
 * <p>
 * If a Frame, or Frame subclass, is not avialable call with:
 * <p>
 * <code>FontAndColorChooser fc = new FontAndColorChooser(null, true/false);</code>
 * <p>
 * If a frame is provided, the dialog is centered over that frame.  Otherwise,
 * the dialog is positioned 20 pixels down and to the right of the upper left
 * corner of the display screen.
 * <p>
 * If the user presses the Cancel button all of the getter methods will return
 * <code>null</code>.  If the user presses the OK button without selecting a
 * color or font the <code>getFont</code> will return a color of black
 * (r=0,g=0,b=0) and getFont will return a system default font such as:
 * <p>
 * javax.swing.plaf.FontUIResource[family=Tahoma,name=Tahoma,style=plain,size=13]
 * <p>
 *Example usage:
 * <p>
 * FontAndColorChooser fc = new FontAndColorChooser(this);
 * <p>
 * Font font = fc.getNewFont();
 * <p>
 * Color color = fc.getNewColor();
 * <p>
 * AttributeSet atts = fc.getAttributes();
 * <p>
 * System.out.println("F&C -toString = " + fc.toString());
 * <p>
 * @param parent The Frame, or subclass, from which the dialog was called.
   * This dialog will be centered on the parent.  If
   * a <code>null</code> is passed as the parent the dialog will be positioned
   * 20 pixels down and to the right of the upper left corner of the display.
 */
  public FontAndColorChooser(Frame parent) {
    super(parent, "Font & Color Chooser", true);
    attributes = new SimpleAttributeSet();
    this.parent = parent;

    // make sure that any way they cancel the window does the right thing
    addWindowListener(new WindowAdapter() {
    public void windowClosing(WindowEvent e) {
      closeAndCancel(); }});

    // Start the long process of setting up our interface
    Container c = getContentPane();

    JPanel fontPanel = new JPanel();

    // Get the fonts which are available on the system.
    GraphicsEnvironment g;
    g = GraphicsEnvironment.getLocalGraphicsEnvironment();
    String[] fonts;
    fonts = g.getAvailableFontFamilyNames();
    fontName = new JComboBox(fonts);

    fontName.setSelectedIndex(0);
    fontName.addActionListener(this);
    fontSize = new JTextField("12", 4);
    fontSize.setHorizontalAlignment(SwingConstants.RIGHT);
    fontSize.addActionListener(this);
    fontBold = new JCheckBox("Bold");
    fontBold.setSelected(true);
    fontBold.addActionListener(this);
    fontItalic = new JCheckBox("Italic");
    fontItalic.addActionListener(this);

    JLabel         alphaLabel     = new JLabel("Alpha (1-255)");
    alphaTextField.setText("255");
    alphaTextField.setColumns(3);
    alphaTextField.setActionCommand("alphaTextField");
    alphaTextField.addActionListener(this);

    fontPanel.add(fontName);
    fontPanel.add(new JLabel(" Size: "));
    fontPanel.add(fontSize);
    fontPanel.add(fontBold);
    fontPanel.add(fontItalic);
    fontPanel.add(alphaLabel);
    fontPanel.add(alphaTextField);

    c.add(fontPanel, BorderLayout.NORTH);

    // Set up the color chooser panel and attach a change listener so that color
    // updates get reflected in our preview label.
    colorChooser = new JColorChooser(Color.black);
    colorChooser.getSelectionModel()
                .addChangeListener(new ChangeListener() {
    public void stateChanged(ChangeEvent e) {
        updatePreviewColor(); } });
    c.add(colorChooser, BorderLayout.CENTER);

    JPanel previewPanel = new JPanel(new BorderLayout());
    previewLabel = new JLabel("Here's a sample of this font.");
    previewLabel.setForeground(colorChooser.getColor());
    previewPanel.add(previewLabel, BorderLayout.CENTER);

    // Add in the Ok and Cancel buttons for our dialog box
    JButton okButton = new JButton("Ok");
    okButton.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent ae) {
        closeAndSave();
      }
    });
    JButton cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(new ActionListener() {
     public void actionPerformed(ActionEvent ae) {
        closeAndCancel();
      }
    });

    JPanel controlPanel = new JPanel();
    controlPanel.add(okButton);
    controlPanel.add(cancelButton);
    previewPanel.add(controlPanel, BorderLayout.SOUTH);

    // Give the preview label room to grow.
//    previewPanel.setMinimumSize(new Dimension(100, 100));
//    previewPanel.setPreferredSize(new Dimension(100, 100));

    c.add(previewPanel, BorderLayout.SOUTH)
        ;
   // Position the dialog
   if (parent == null)
     { // If there is no parent then position the dialog at the upper left
     this.setLocation(20, 20);
     }
   else
     { // Else position the dialog over the center of the parent
     Dimension dlgSize = this.getPreferredSize();
     Dimension frmSize = parent.getSize();
     Point loc = parent.getLocation();
     this.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                      (frmSize.height - dlgSize.height) / 2 + loc.y);
     }
  this.pack();
  this.setVisible(true);
  } // End FontAndColorChooser



  // Ok, something in the font changed, so figure that out and make a
  // new font for the preview label
  public void actionPerformed(ActionEvent ae) {
    // Check the name of the font
    if (!StyleConstants.getFontFamily(attributes)
                       .equals(fontName.getSelectedItem())) {
      StyleConstants.setFontFamily(attributes,
                                   (String)fontName.getSelectedItem());
    }
    // Check the font size (no error checking yet)
    if (StyleConstants.getFontSize(attributes) !=
                                   Integer.parseInt(fontSize.getText())) {
      StyleConstants.setFontSize(attributes,
                                 Integer.parseInt(fontSize.getText()));
    }
    // Check to see if the font should be bold
    if (StyleConstants.isBold(attributes) != fontBold.isSelected()) {
      StyleConstants.setBold(attributes, fontBold.isSelected());
    }
    // Check to see if the font should be italic
    if (StyleConstants.isItalic(attributes) != fontItalic.isSelected()) {
      StyleConstants.setItalic(attributes, fontItalic.isSelected());
    }
    // and update our preview label
    updatePreviewFont();
  }

  // Get the appropriate font from our attributes object and update
  // the preview label
  protected void updatePreviewFont() {
    String name = StyleConstants.getFontFamily(attributes);
    boolean bold = StyleConstants.isBold(attributes);
    boolean ital = StyleConstants.isItalic(attributes);
    int size = StyleConstants.getFontSize(attributes);

    //Bold and italic don't work properly in beta 4.
    Font f = new Font(name, (bold ? Font.BOLD : 0) +
                            (ital ? Font.ITALIC : 0), size);
    previewLabel.setFont(f);
    updatePreviewColor();
  }

  // Get the appropriate color from our chooser and update previewLabel
  protected void updatePreviewColor() {
    newColor = colorChooser.getColor();
    // Extract the basic colors for use in building the color with alpha.
    int red   = newColor.getRed();
    int green = newColor.getGreen();
    int blue  = newColor.getBlue();
    // Now build new color using the rgb and alpha values
    int newAlpha = 255;
    String alpha = alphaTextField.getText();
    newAlpha = Integer.valueOf(alpha);
    // The alpha value must be between 1 and 255
    if (newAlpha < 1 || newAlpha > 255)
      {
      newAlpha = 255;
      alphaTextField.setText("255");
      JOptionPane.showMessageDialog(parent,
             "The alpha transparency value must be between 1 and 255." +
           "\n        The program has reset the alpha to 255.",
           "The alpha value was in error.",
           JOptionPane.WARNING_MESSAGE);
      }

    newAlphaColor = new Color(red,green,blue,newAlpha);

    previewLabel.setForeground(newAlphaColor);

    Graphics g = previewLabel.getGraphics();
    previewLabel.update(g);
    // manually force the label to repaint
    previewLabel.repaint();
  }

  private void closeAndSave()
    {
    newColor = previewLabel.getForeground();
    newFont = previewLabel.getFont();
    setVisible(false);
    }

  private void closeAndCancel()
    {
    // erase any font information and then close the window
    newColor = null;
    newFont = null;
    attributes = null;
    setVisible(false);
    }

  /**
   * Returns the color and transparency settings which were selected by the user.
   *
   * @return Color object
   */
  public Color getNewColor()
    { return newColor; }

  /**
   * Returns the font, bold and italic settings selected by the user.
   *
   * @return Font object
   */
  public Font getNewFont()
    { return newFont; }

  /**
   * Returns the font attribute set.
   *
   * @return The font attribute set.
   */
  public AttributeSet getAttributes()
    { return attributes; }

  /**
   * Gets the font attribute set.
   *
   * @return The font attribute set.  For example:
   * <p>
   * Font=java.awt.Font[family=AR BERKLEY,name=AR BERKLEY,style=bolditalic,size=12]
   * newColor=java.awt.Color[r=255,g=0,b=0]
   * Attributes+bold=true italic=true family=AR BERKLEY
   */
  public String toString()
    { return "Font="+newFont+" newColor="+newColor+" Attributes+"+attributes; }

} // End FontAndColorChooser
