package fwpUtil;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
/**
 * Title: Font chooser dialog.</p>
 * <p>
 * Description: A font chooser dialog which suports setting the size, bold
 * and italic properties of the font.
 * <p>
 * Copyright: Copyright (c) 2011
 * @author Fred Pospeschil - This file is placed in the public domain
 * and can be used and adapted without restrictions.  The user of the code is
 * totally responsible for insuring its suitability in the intended application.
 * @version 1.0 - 2011
 * <p>
 */

public class FontChooser extends JDialog implements ActionListener {

  private static JComboBox          fontName;
  private static JCheckBox          fontBold, fontItalic;
  private static JTextField         fontSize;
  private static JLabel             previewLabel = new JLabel
                                             ("Here's a sample of this font.");
  private static SimpleAttributeSet attributes= new SimpleAttributeSet();
  private static Font               newFont = new Font("Arial",0,12);

  /**
   * Displays a dialog in which the user can select a font.
   * <p>
   * A font chooser that allows users to pick a font by name, set the size and
   * select whether the font should be bold and/or italic.
   * <p>
   * It can be called with <code>FontChooser fc = new FontChooser(this);</code>
   * if a frame is avialable or it can be called with:
   * <code>FontChooser fc = new FontChooser(null);</code> if a frame is not
   * available.
   * <p>
   * This dialog is an adaptation of the font chooser presented in Java Swing by
   * Robert Eckstein, Marc Loy & Dave Wood, O'Reilly, Sebastopol, CA 1998,
   * ISBN 1-56592-455-X. See pages 381-387.  The fixed list of fonts approach
   * used by the O'Reilly example was replaced by the more general getting a
   * list of all available fonts as described on pages 788-791 of Java for
   * Dummies by Doug Lowe and Barry Burd, Wiley Publishing, Inc. 2007.
   * @param parent The parent Frame from which this dialog was called.  If the
   * dialod is called with a valid parent the dialog will be positioned over
   * the center of the parent.  If a <code>null</code> is passed instead of a
   * parent then the dialog will be located at the upper left of the screen.
   * */
  public FontChooser(Frame parent)
    {
    super(parent, "Color Chooser", true);
    setTitle("Font Chooser");

    // make sure that any way they cancel the window does the right thing
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        closeAndCancel(); }});

    // Start the long process of setting up our interface
    Container c = getContentPane();
    JPanel fontPanel = new JPanel();
    // Build the font combo box
    GraphicsEnvironment g;
    g = GraphicsEnvironment.getLocalGraphicsEnvironment();
    String[] fonts;
    fonts = g.getAvailableFontFamilyNames();
    fontName = new JComboBox(fonts);

    fontName.setSelectedIndex(0);
    fontName.addActionListener(this);
    fontSize = new JTextField("12", 4);
    fontSize.setHorizontalAlignment(SwingConstants.RIGHT);
    fontSize.addActionListener(this);
    fontBold = new JCheckBox("Bold");
    fontBold.setSelected(true);
    fontBold.addActionListener(this);
    fontItalic = new JCheckBox("Italic");
    fontItalic.addActionListener(this);

    fontPanel.add(fontName);
    fontPanel.add(new JLabel(" Size: "));
    fontPanel.add(fontSize);
    fontPanel.add(fontBold);
    fontPanel.add(fontItalic);

    c.add(fontPanel, BorderLayout.NORTH);


    JPanel previewPanel = new JPanel(new BorderLayout());
    previewPanel.add(previewLabel, BorderLayout.CENTER);

    // Add in the Ok and Cancel buttons for our dialog box
    JButton okButton = new JButton("Ok");
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        closeAndSave();
      }
    });
    JButton cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        closeAndCancel();
      }
    });

    JPanel controlPanel = new JPanel();
    controlPanel.add(okButton);
    controlPanel.add(cancelButton);
    previewPanel.add(controlPanel, BorderLayout.SOUTH);

    // Give the preview label room to grow.
    previewPanel.setMinimumSize(new Dimension(100, 100));
    previewPanel.setPreferredSize(new Dimension(100, 100));

    c.add(previewPanel, BorderLayout.SOUTH);

   // Position the dialog
   if (parent == null)
     { // If there is no parent then position the dialog at the upper left
     this.setLocation(20, 20);
     }
   else
     { // Else position the dialog over the center of the parent
     Dimension dlgSize = this.getPreferredSize();
     Dimension frmSize = parent.getSize();
     Point loc = parent.getLocation();
     this.setLocation( (frmSize.width - dlgSize.width) / 2 + loc.x,
                      (frmSize.height - dlgSize.height) / 2 + loc.y);
     }
  this.pack();
  this.setVisible(true);

  } // End FontChooser constructor



  // Ok, something in the font changed, so figure that out and make a
  // new font for the preview label
  public void actionPerformed(ActionEvent ae)
   {
    // Check the name of the font
    if (!StyleConstants.getFontFamily(attributes)
                       .equals(fontName.getSelectedItem())) {
      StyleConstants.setFontFamily(attributes,
                                   (String)fontName.getSelectedItem());
    }
    // Check the font size (no error checking yet)
    if (StyleConstants.getFontSize(attributes) !=
                                   Integer.parseInt(fontSize.getText())) {
      StyleConstants.setFontSize(attributes,
                                 Integer.parseInt(fontSize.getText()));
    }
    // Check to see if the font should be bold
    if (StyleConstants.isBold(attributes) != fontBold.isSelected()) {
      StyleConstants.setBold(attributes, fontBold.isSelected());
    }
    // Check to see if the font should be italic
    if (StyleConstants.isItalic(attributes) != fontItalic.isSelected()) {
      StyleConstants.setItalic(attributes, fontItalic.isSelected());
    }
    // and update our preview label
    updatePreviewFont();
  } // End actionPerformed(ActionEvent ae)



  // Get the appropriate font from our attributes object and update
  // the preview label
  protected void updatePreviewFont()
    {
    String name = StyleConstants.getFontFamily(attributes);
    boolean bold = StyleConstants.isBold(attributes);
    boolean ital = StyleConstants.isItalic(attributes);
    int size = StyleConstants.getFontSize(attributes);

    //Bold and italic don't work properly in beta 4.
    Font f = new Font(name, (bold ? Font.BOLD : 0) +
                            (ital ? Font.ITALIC : 0), size);
    previewLabel.setFont(f);
    } // End updatePreviewFont

  private void closeAndSave()
    {
    setVisible(false);
    newFont = previewLabel.getFont();
    }

  private void closeAndCancel()
    { setVisible(false); }

  /**
   * Returns the font and settings selected by the user.
   *
   * @return Font object
   * <p>
   * Usage example:
   * FontChooser dlg = new FontChooser(this);
   * Font font = dlg.getNewFont();
   * System.out.println("font=" + font);
   */
  public Font getNewFont()
    { return newFont; }

  /**
   * Returns the font attributes.
   *
   * @return new font attributes.  For example:
   * <p>
   * javax.swing.plaf.FontUIResource[family=Tahoma,name=Tahoma,style=plain,size=13]
   */
  public String toString()
    { return "" + newFont; }

} // End FontChooser
