package fwpUtil;

import java.io.FileOutputStream;
import java.io.PrintStream;

/**
 * Title: Logger.</p>
 * <p>
 * Description: A basic logger capability which uses the same logging levels
 * as the much larger standard Java logger.
 * <p>
 * Copyright: Copyright (c) 2011
 * @author Fred Pospeschil - This file is placed in the public domain
 * and can be used and adapted without restrictions.  The user of the code is
 * totally responsible for insuring its suitability in the intended application.
 * @version 1.0 - 2011
 */

public class Logger
  {
  private static FileOutputStream fos;
  private static PrintStream ps;

  // Logging control variables
  private static int    logLevel;
  private static String logFilePath;
  private static String logFileName;
  private static String logItHelpPath;
  private static String logItHelpFileName;

  // Constant definitions for the logging levels
  public static final int OFF     = 9;
  public static final int SEVERE  = 8;
  public static final int WARNING = 7;
  public static final int INFO    = 6;
  public static final int CONFIG  = 5;
  public static final int FINE    = 4;
  public static final int FINER   = 3;
  public static final int FINEST  = 2;
  public static final int ALL     = 1;

/**
 * Creates a program flow and data logging capability.
 * <p>
 * When activated, the Logger directs all logging messages as well as all
 * System.out and System.err output to a disk based text file.  Program
 * generated log messages and exception stack traces are also directed to the
 * file in the exact order they were generated.  Nine levels of logging are
 * supported: Logger.OFF, Logger.SEVERE, Logger.WARNING, Logger.INFO,
 * Logger.CONFIG, Logger.FINE, Logger.FINER, Logger.FINEST, and Logger.ALL.
 * <p>
 * The log file name and path are set by the parameters passed by the calling
 * method.
 * <p>
 * The Logger is started with:
 * <p>
 * Logger.startLogger(logPath, LogName, helpPath, helpName);
 * <p>
 * Set a new logging level with : Logger.setLogLevel(logger.CONFIG);
 *<p>
 * When a writeLog call is made, the value of the level parameter is tested
 * against the value passed in the last call to setLogLevel.  If the value is
 * greater, then the log message is written to the log file.  In the list above,
 * the logging levels are listed in decreasing order were OFF has the
 * greatest value and ALL has the lowest.
 * <p>
 * @param filePath The path to the directory which is to hold the log file.
 * This string must end with a file separator appropriate for the system on
 * which the program is running.  For example:
 * String logPath = System.getProperty("user.dir")
 *                    + System.getProperty("file.separator");
 * <p>
 * @param logName The name of the log file.
 * For example: String logFileName = "Log.txt";
 * <p>
 * @param helpPath The path to the LogIt htm help file.  Unless there is
 * a reason not to, the help file could easily go into the same
 * directory as the log file.
 * <p>
 * @param helpName The name of the LogIt htm help file.
 * For example: String helpFileName = "LogItHelp.htm"; // The default name
 * <p>
 * Example usage: The following code can be copied into program initialization
 * area and it will do everyting necessary to get the logger going.  The log
 * file will be placed in the directory where the program is located.  The
 * default LogIt help file () should also be placed it that directory.  These
 * files do not need to be co-located and can be placed anywhere.
 * <p>
 * See the Logger documentation in the fwpUtil directory for the example.
   */
  public static void startLogger(String logPath,  String logName,
                                 String helpPath, String helpName)
  {
  try
    {
    logFilePath       = logPath;
    logFileName       = logName;
    logItHelpPath     = helpPath;
    logItHelpFileName = helpName;
    // Create an output stream/file.  The "false" parm says to make
    // a new file rather than appending to an existing file.  i.e,
    // erase the old file if one exists with the same name.
    // The following is based on www.devx.com/tips/Tip/5616 by Brett spell
    fos = new FileOutputStream(logFilePath + logFileName, false);
    ps = new PrintStream(fos);
    // Redirect output of System.Out and System.Err to a disk file
    System.setOut(ps);
    System.setErr(ps);
//    logLevel = OFF;
    }
  catch (Exception exception)
    { exception.printStackTrace(); }

  // Initialize the log file
  System.out.println("Default LogFile = " + logFilePath + logFileName);
  } // END startLogger()

//#########################################################################
    /**
     * Gets the currently set log file name.
     * @return The log file name.
     */
    public static String getLogFileName()
    {
    writeLog(FINER,"Logger getLogFileName = " + logFileName);
    return logFileName;
    }

    /**
     * Gets the currently set log file path.
     * @return The log file path.
     */
    public static String getLogFilePath()
    {
    writeLog(FINER,"Logger getLogFilePath = " + logFilePath);
    return logFilePath;
    }

    /**
     * Gets the log help file name.
     * @return The log file name.
     */
    public static String getlogItHelpFileName()
    {
    writeLog(FINER,"Logger getlogItHelpFileName = " + logItHelpFileName);
    return logItHelpFileName;
    }

  /**
   * Gets the LogIt help file path.
   * @return The LogIt help file path.
   */
  public static String getLogItHelpPath()
    {
    writeLog(FINER,"Logger getLogItHelpPath = " + logItHelpPath);
    return logItHelpPath;
    }

    /**
     * Gets the current logging level.
     * <p>
     * @return Logger.OFF, Logger.SEVERE, Logger.WARNING, Logger.INFO,
     * Logger.CONFIG, Logger.FINE, Logger.FINER, Logger.FINEST, and Logger.ALL.
     */
    public static int getLogLevel()
    {
    writeLog(FINER,"Logger getLogLevel= " + logLevel);
    return logLevel;
    }

    /**
     * Sets the log file name.
     * @param fileName The log file name.
     */
    public static void setLogFileName(String fileName)
    {
    logFileName = fileName;
    writeLog(FINER,"Logger setLogFileName = " + logFileName);
    }

    /**
     *Sets the log file path.
     * @param pathName A String defining the path to the log file.  This string
     * must end with a file separator.
     */
    public static void setLogFilePath(String pathName)
    {
    logFilePath = pathName;
    writeLog(FINER,"Logger setLogFilePath = " + logFilePath);
    }

    /**
     * Sets the log help file name.
     * @param fileName The log help file name.
     */
    public static void setlogItHelpFileName(String fileName)
    {
    logItHelpFileName = fileName;
    writeLog(FINER,"Logger setLogFileName = " + logItHelpFileName);
    }

  /**
   *Sets the LogIt help file path.
   * @param pathName A String defining the path to the LogIt help file.
   * This string must end with a file separator.
   */
  public static void setLogItHelpPath(String pathName)
    {
    logItHelpPath = pathName;
    writeLog(FINER,"Logger setLogItHelpPath = " + logItHelpPath);
    }

  /**
   *Sets the logging level.
   * <p>
   * @param level Logger.OFF, Logger.SEVERE, Logger.WARNING, Logger.INFO,
   * Logger.CONFIG, Logger.FINE, Logger.FINER, Logger.FINEST, and Logger.ALL.
   */
   public static void setLogLevel(int level)
    {
    logLevel = level;
    writeLog(FINER,"Logger setLogLevel = " + logLevel);
    }

    /**
     * Write a message to the log file if the level parameter is greater than
     * the currently set logging level.
     * <p>
     * @param level Logger.OFF, Logger.SEVERE, Logger.WARNING, Logger.INFO,
     * Logger.CONFIG, Logger.FINE, Logger.FINER, Logger.FINEST, and Logger.ALL.
     * <p>
     * @param textessage The message to be written to the log file.
     */
    public static void writeLog(int level, String textMessage)
    {
    if (level >= logLevel)
      {
      System.out.println(textMessage) ;
      }
    } // END writeLog

  /**
   * Returns the name of the class.
   *
   * @return The class name, log file path+name, log level and log help file
   * path+name.  For example:
   * <p>
   * Logger LogFile=D:\A JOptionPanel\Log.txt Log Level=1
   *  HelpFile=D:\A JOptionPanel\LogItHelp.htm
   */
  public String toString()
    { return "Logger LogFile="+Logger.getLogFilePath()
                              +Logger.getLogFileName()
                              + " Log Level="
                              + Logger.getLogLevel()
                              + " HelpFile="
                              + Logger.getLogItHelpPath()
                              + Logger.getlogItHelpFileName();
    }

} // END Logger class
