package fwpUtil;

import java.awt.*;
import javax.swing.*;
import java.awt.BorderLayout;
import fwpUtil.Logger;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.*;
import java.awt.Font;
import java.awt.Insets;

/**
 * Title: An basic text file viewer.
 * </p>
 * Description: This code displays a basic text viewer with a vertical scroll
 * bar.  Its purpose is to display standard text help files from within
 * an application program.
 * <p>
 * Copyright: Copyright (c) 2011
 * <p>
 * Company: This code is placed in the public domain for un-restricted use.
 * The user is totally responsible for verifying the correct operation of
 * the code, or adapted code, in their application.
 * @author Fred Pospeschil
 * @version 1.0
  * <p>
 * @see Robert Eckstein et al., Java Swing, (Sebastopol, CA, OReilly
 * & Associates, Inc., 1998) pp 712-720
 */
public class TextViewer
    extends JDialog
  {
  private JPanel       panel1        = new JPanel();
  private BorderLayout borderLayout1 = new BorderLayout();
  private JScrollPane  jScrollPane   = new JScrollPane();
  private JButton      jButtonDone   = new JButton();
  private JPanel       jPanelSouth   = new JPanel();
  private JTextArea    jTextArea     = new JTextArea();
  private String       filePath;
  private String       fileName;


//##############################################################################
/**
 * A basic viewer for disk based text files.  This version is called by
 * other than a <code>Dialog</code> or <code>Frame</code>object.
 * The constructor concatinates the filePath + fileName strings to form
 * the full path\filename string which is used to load the text file.
 * It's intended purpose is for viewing local disk based text files.
 * <p>
 * The viewer is non-modal so the user can scroll/read the contents while still
 * working with the application program.
 * <p>
 * The filePath and fileName are displayed as the title of the viewer.
 * <p>
 * When building the filePath string, suggest using:
 *<p>
 * filePath = System.getProperty("user.dir") +"\\";  or
 *<p>
 *filePath = System.getProperty("user.dir")
 *                            + System.getProperty("file.separator");
 *<p>
 *A valid path\filename would be C:\MicroMap\MicorMapLog.txt
 *<p>
 * @param filePath The full path to the text file ending with a file seperator.
 *<p>
 * @param fileName The full file name with a .txt extension
 */
  public TextViewer(String filePath, String fileName)
    {
    super.setTitle( filePath + fileName);
    try
      {
      Logger.writeLog(Logger.CONFIG, "TextViewer constructor entered");
      setDefaultCloseOperation(DISPOSE_ON_CLOSE);
      jbInit(filePath, fileName);
      }
    catch (Exception exception)
      { exception.printStackTrace(); }
    Logger.writeLog(Logger.CONFIG, "TextViewer constructor exited");
    } // END TextViewer for other than Dialogs and Frames

//##############################################################################
  private void jbInit(String filePath,String fileName) throws Exception
    {
    // JBuilder generated GUI code
    Logger.writeLog(Logger.CONFIG, "TextViewer jbInit entered");
    panel1.setLayout(borderLayout1);
    jButtonDone.setFont(new java.awt.Font("Tahoma", Font.BOLD, 12));
    jButtonDone.setText("Done");
    jButtonDone.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e)
        { jButtonDone_actionPerformed(e); }
      });
    jPanelSouth.setFont(new java.awt.Font("Tahoma", Font.BOLD, 12));
    jTextArea.setFont(new java.awt.Font("Dialog", Font.PLAIN, 14));
    jTextArea.setMargin(new Insets(2, 2, 2, 2));
    jTextArea.setLineWrap(true);
    jScrollPane.setVerticalScrollBarPolicy(JScrollPane.
        VERTICAL_SCROLLBAR_ALWAYS);
    getContentPane().add(panel1);
    panel1.add(jScrollPane, java.awt.BorderLayout.CENTER);
    jPanelSouth.add(jButtonDone);
    panel1.add(jPanelSouth, java.awt.BorderLayout.SOUTH);
    jScrollPane.getViewport().add(jTextArea);
    // End of JBuilder generated GUI code

    Logger.writeLog(Logger.FINE, "TextViewer file load started. ");
    Logger.writeLog(Logger.FINE, "Loading file " + filePath + fileName);

    try
       {
       // filePath = "dummy\\";  // insert IO error for testing purposes
       // Create and open the text input file
       File f = new File(filePath, fileName);
       FileReader in = new FileReader(f);
       char[] buffer = new char[4096];
       int len;

       while(( len = in.read(buffer)) != -1) // While not end-of-file
         {
         String input = new String(buffer, 0, len); // Read from input file
         jTextArea.append(input);  // Stuff the text into the textArea
         }// END while
       in.close();  // All done - close the input file
       pack();                    // Pack the dialog window
       setLocation(20, 20);
       setSize(500, 400);         // Keep the size of the widow reasonably small
       setVisible(true);          // Now, show the darn thing
       Logger.writeLog(Logger.CONFIG, "TextViewer file load ended.");
       } // END try
     catch (IOException e)
       {
       Logger.writeLog(Logger.SEVERE, "TextViewer jbInit IOException entered");
       Logger.writeLog(Logger.SEVERE, "In TextViewer -  Unable to open file "
                                     + filePath + fileName);
       JOptionPane.showMessageDialog(null,
                                     "Unable to open file " + filePath + fileName,
                                     "File Open Error",
                                     JOptionPane.ERROR_MESSAGE);
       e.printStackTrace();
       } // END catch
  Logger.writeLog(Logger.CONFIG, "TextViewer jbInit exited");
  } // END  JbInit

//##############################################################################
  private void jButtonDone_actionPerformed(ActionEvent e)
    {
    Logger.writeLog(Logger.CONFIG, "TextViewer jButtonDone_actionPerformed");
    dispose();
    }

//##############################################################################
  /**
   * Returns the name of the class.
   *
   * @return The name of the class - textViewer  plus the path and file
   * name of the textViewer help file.
   */
  public String toString()
    { return "textViewer " + filePath + " " + fileName; }


} // END class TextViewer
